
if exists (select 'x' from information_schema.routines where routine_name = 'sync_LoadParallelEntityScopeModel')
drop procedure dbo.sync_LoadParallelEntityScopeModel
go

create procedure [dbo].sync_LoadParallelEntityScopeModel 
(
	@p_TargetModel int, 
	@p_EntityIdHashKey nvarchar(32)
)
as
set nocount on 
begin	
	/*
	The proc is called on both the sending and receiving database before ANY esync activity happens since v8.6.1.
	It backs up the rows in the base scope tables to the locker tables and if needed will load the right scope model
	rows from the locker tables to the base scope tables.

	Anything other than zero as a return value indicates a problem:

		Values returned between -10 and -1 are application defined errors capturing problems that are forseeable.
			Example: -1 is the error returned if the value supplied to @p_TargetModel is invalid.
		 
		Values less than -10 are actually positive database error codes and should be debugged as such.
			Example: -2627 is actually DB ERROR 2627 (Primary key violation). 
	*/
	begin transaction;
	begin try

	declare @result int;
	declare @steperror int;
	declare @v_TargetModel table (TargetModelId int, TargetHashCd nvarchar(32));
	declare @v_ErrorLog table
		(	scope_chglog_id uniqueidentifier default(newid()) not null,
			scope_chglog_procnm nvarchar(128) null,
			scope_chglog_target_from int null,
			scope_chglog_target_from_hashcd nvarchar(38) null,
			scope_chglog_target_to int null,
			scope_chglog_target_to_hashcd nvarchar(32) null,
			scope_chglog_step_cd int not null,
			scope_chglog_step_nm nvarchar(128) not null,
			scope_chglog_proc_cd int null,
			scope_chglog_error_cd int null,
			scope_chglog_error_msg nvarchar(2048) not null,
			scope_chglog_dtm datetime default getdate() not null,
			scope_chglog_row_cnt int null
		);
	declare @v_procname nvarchar(128);
	declare @v_stepname nvarchar(128);
	declare @v_errorcd int;
	declare @v_errormsg nvarchar(2048);
	declare @v_EntityIdHashKey nvarchar(32);
	declare @v_rows int;
	declare @v_CurrentlyLoadedScopeRows int;
	declare @v_DetailMessage nvarchar(2048);
	declare @v_ScopesWithinModel TABLE(scopename nvarchar(100));
	
	set @v_procname = OBJECT_NAME(@@PROCID);
	set @v_EntityIdHashKey = @p_EntityIdHashKey;
			
	insert into @v_TargetModel (TargetModelId, TargetHashCd )
	select @p_TargetModel,@v_EntityIdHashKey;

	if (@p_TargetModel = 1)
		BEGIN
			--for master scope model, the model name is 'master' unlike other scope models. this is the only
			--case where one model contains multiple scopes.
			insert into @v_ScopesWithinModel (scopename) values ('aspnet');
			--the custom scope is the only 'non static' scope which can change if the user wants it to change.
			insert into @v_ScopesWithinModel (scopename) values ('custom');
			insert into @v_ScopesWithinModel (scopename) values ('ektron');
			insert into @v_ScopesWithinModel (scopename) values ('history');
			insert into @v_ScopesWithinModel (scopename) values ('notification');
			insert into @v_ScopesWithinModel (scopename) values ('search');
			insert into @v_ScopesWithinModel (scopename) values ('workflow');
		END
	else
		BEGIN
			insert into @v_ScopesWithinModel
			select name from scope_locker_manager
			where id = @p_TargetModel;
		END
	
	set @result = 0;
		
		/*  1. detect if model exists in scope_locker_manager
			2. find record in scope_locker_history indicating which scope is currently active
			3. take current data and save it back to the locker for the correct type 
			4. delete current data from tables
			5. restore type of data selected from lockers into the tables 
			6. insert new row into scope_locker_history 
		*/
	
	/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		Step 1: Check that the target exists

		Check that the target exists, if not log it and exit with a result of -1
	 
	  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/
	set @v_stepname='Step1: Check target exists';
	
	if not exists(select 'X' from scope_locker_manager where id = @p_TargetModel) 
	begin
		set @result = -1;
		
		--exec dbo.sync_LoadParallelLogging @p_procname=@v_procname,@p_stepname=@v_stepname,@p_error_msg='success',@p_target_to=@p_TargetModel,@p_step_cd=@result,@p_proc_cd=@result,
		--	@p_error_msg='skipped - Target Model does not exist';
			
		insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
								 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
								 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
		values (@v_procname, 0, @p_TargetModel, @result, @v_stepname, @result, @result, 'skipped - Target Model does not exist' ,NULL,@v_EntityIdHashKey);
	
		IF @@TRANCOUNT > 0
		    COMMIT TRANSACTION;
			
		insert into dbo.scope_chglog_tbl 
		  	   (scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
			    scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
			    scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
			    scope_chglog_dtm, scope_chglog_row_cnt)
		select scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
		        scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
		        scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
		        scope_chglog_dtm, scope_chglog_row_cnt
	from @v_ErrorLog
	order by scope_chglog_dtm asc;
	
		return @result;
	end;
			
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, 0, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,NULL, @v_EntityIdHashKey);

	--scope_info table should not have any default rows. These rows are only created when sync runs.
	delete from scope_info_locker where isdefault = 1;

	/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		Step2: Backup scopes from base tables to lockers
		       Whether the currentl scope model is the same as the target model OR NOT we still backup the rows in either case.
			   No need to duplicate the code for this.
       !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/
	set @steperror = -2;
	set	@v_stepname='Step2: Backup scopes';

	--variable name changed. a scope model contains scopes. it is not the same as a scope.
	declare @v_currentlyActiveScopeModel int;
	declare @v_currentlyActiveScopeHash nvarchar(32);
	
	select top 1 @v_currentlyActiveScopeModel = scope_id, @v_currentlyActiveScopeHash = scope_filter_hashcd from scope_locker_history order by date_loaded desc;
	set @v_CurrentlyLoadedScopeRows = @@rowcount;
	
	if (@v_currentlyActiveScopeModel is not null) --if anything is currently loaded.
	BEGIN		
		--backup scope_config
		--hashcd does not matter for scope_config. 
		delete from scope_config_locker where scope_manager_id = @v_currentlyActiveScopeModel and IsDefault = 0;

		--scope_name will not be equivalent to scope_locker_manager.name for anything within the master scope model
		insert into scope_config_locker (scope_manager_id, scope_filter_hashcd, scope_name, scope_table, scope_description, scope_sync_on, scope_after_event)
		select @v_currentlyActiveScopeModel, @v_currentlyActiveScopeHash, scope_name, scope_table, scope_description, 1, scope_after_event 
		from scope_config;

		--backup scope_info
		--moved to using delete-insert, as it is sometimes updating data in wrong rows with older logic (ex: hashcd in ektron scope row).
		delete from scope_info_locker where scope_manager_id = @v_currentlyActiveScopeModel and IsDefault = 0 and coalesce(scope_filter_hashcd,'0') = coalesce(@v_currentlyActiveScopeHash,'0');

		--scope_name will not be equivalent to scope_locker_manager.name for anything within the master scope model
		insert into scope_info_locker (scope_manager_id, scope_filter_hashcd, sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_cleanup_timestamp, scope_restore_count)
		select @v_currentlyActiveScopeModel, @v_currentlyActiveScopeHash, scope_info.sync_scope_local_id, scope_info.sync_scope_id, scope_info.sync_scope_name, scope_info.sync_scope_knowledge, 
			scope_info.sync_scope_cleanup_knowledge, scope_info.sync_scope_datetime, scope_info.sync_scope_timestamp, scope_info.scope_cleanup_timestamp, scope_info.scope_restore_count 
		from scope_info;

		--backup scope_info_history
		--we don't load the rows anymore. hence all rows created are new rows. 
		insert into scope_info_history_locker (scope_manager_id, scope_filter_hashcd, sync_scope_historyid, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_schema, sync_scope_history_datetime)
		select @v_currentlyActiveScopeModel, @v_currentlyActiveScopeHash, scope_info_history.sync_scope_historyid, scope_info_history.sync_scope_id, scope_info_history.sync_scope_name, scope_info_history.sync_scope_knowledge, 
			scope_info_history.sync_scope_cleanup_knowledge, scope_info_history.sync_scope_schema, scope_info_history.sync_scope_history_datetime 
		from scope_info_history;

		--backup scope_loginfo
		--we don't load the rows anymore. hence all rows created are new rows. 
		insert into scope_loginfo_locker (scope_manager_id, scope_filter_hashcd, sync_log_id, sync_scope_id, sync_start_version, sync_end_version, sync_provider_type, sync_changes_total, 
				sync_changes_applied, sync_changes_failed, sync_user_id, date_created, sync_start_time, sync_end_time, sync_scope_position, sync_scope_conflict, sync_profile_id, 
				sync_scope_direction, sync_conflict_policy)
		select @v_currentlyActiveScopeModel, @v_currentlyActiveScopeHash, scope_loginfo.sync_log_id, scope_loginfo.sync_scope_id, scope_loginfo.sync_start_version, scope_loginfo.sync_end_version, 
				scope_loginfo.sync_provider_type, scope_loginfo.sync_changes_total, scope_loginfo.sync_changes_applied, scope_loginfo.sync_changes_failed, scope_loginfo.sync_user_id, 
				scope_loginfo.date_created, scope_loginfo.sync_start_time, scope_loginfo.sync_end_time, scope_loginfo.sync_scope_position, scope_loginfo.sync_scope_conflict, 
				scope_loginfo.sync_profile_id, scope_loginfo.sync_scope_direction, scope_loginfo.sync_conflict_policy 
		from scope_loginfo;

		--backup scope_table_map - static except for custom scope only.
		if (@v_currentlyActiveScopeModel = 1 and (exists (select 'x' from scope_table_map where scope_name = 'custom')))
		BEGIN
			delete from scope_table_map_locker where scope_name = 'custom';
		
			insert into scope_table_map_locker (scope_manager_id, scope_name, table_name, table_order)
			select @v_currentlyActiveScopeModel, scope_table_map.scope_name, scope_table_map.table_name, scope_table_map.table_order 
			from scope_table_map where scope_name = 'custom';
		END
	END
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);

	
	/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		Step 3: Get current scope model

		Get current scope model that is in use. If it is the same as the target exit as no change is required.	 
	  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/
	set @steperror = -3;
	set	@v_stepname='Step3: Get current scope model';		
	
	if (@v_currentlyActiveScopeModel = @p_TargetModel and coalesce(@v_currentlyActiveScopeHash,'0') = coalesce(@v_EntityIdHashKey,'0') and @v_CurrentlyLoadedScopeRows <> 0) 
	BEGIN
		set @result = 0;

		insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
								 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
								 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
		values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success - Scope Model change skipped as it has not changed.' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
			
		IF @@TRANCOUNT > 0
		    COMMIT TRANSACTION;
			
		insert into dbo.scope_chglog_tbl 
			   (scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
				scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
				scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
				scope_chglog_dtm, scope_chglog_row_cnt)
		select scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
			    scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
			    scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
			    scope_chglog_dtm, scope_chglog_row_cnt
		from @v_ErrorLog
		order by scope_chglog_dtm asc;
		
		return @result;
	END
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
	
	/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		Step4: Remove current scope

	  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/
	set @steperror = -4;
	set	@v_stepname='Step4: Remove current scope';
	
	delete from scope_config;
	delete from scope_info;
	delete from scope_info_history;
	delete from scope_loginfo;
	delete from scope_table_map;
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
	
	/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		Step5: Restore requested target scope
		
	  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/
	set @steperror = -5;
	set	@v_stepname='Step5: Restore requested target scope';
	
	/**************************************
		
		S5: SCOPE_CONFIG RESTORE - static table between scope models.
		
			1. Try non-default rows first. 
			2. Try default 
	
	***************************************/

	set @v_rows = 0;
	
	if (@p_TargetModel <> 1)
		BEGIN
			--for all non master scope models, there is only one row. timestamp does not matter - there is no such column anyway.
			insert into dbo.scope_config ( scope_name, scope_table, scope_description, scope_sync_on, scope_after_event )
			select top 1 scl.scope_name, scl.scope_table, scl.scope_description, scl.scope_sync_on, scl.scope_after_event
			from scope_config_locker scl
			where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.IsDefault = 0;

			set @v_rows = @@rowcount;
		END
	ELSE
		BEGIN
			--ensure that only 1 row per scope gets added. there might be a better way to do this.
			-- timestamp does not matter - there is no such column anyway.

			insert into dbo.scope_config ( scope_name, scope_table, scope_description, scope_sync_on, scope_after_event )
			select top 1 scl.scope_name, scl.scope_table, scl.scope_description, scl.scope_sync_on, scl.scope_after_event
			from scope_config_locker scl
			where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.IsDefault = 0 and
			scl.scope_name = 'aspnet';

			set @v_rows = @@rowcount;

			--this is the only row which may or may not get inserted because it might not be present.
			insert into dbo.scope_config ( scope_name, scope_table, scope_description, scope_sync_on, scope_after_event )
			select top 1 scl.scope_name, scl.scope_table, scl.scope_description, scl.scope_sync_on, scl.scope_after_event
			from scope_config_locker scl
			where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.IsDefault = 0 and
			scl.scope_name = 'custom';

			if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END

			insert into dbo.scope_config ( scope_name, scope_table, scope_description, scope_sync_on, scope_after_event )
			select top 1 scl.scope_name, scl.scope_table, scl.scope_description, scl.scope_sync_on, scl.scope_after_event
			from scope_config_locker scl
			where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.IsDefault = 0 and
			scl.scope_name = 'ektron';

			if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END

			insert into dbo.scope_config ( scope_name, scope_table, scope_description, scope_sync_on, scope_after_event )
			select top 1 scl.scope_name, scl.scope_table, scl.scope_description, scl.scope_sync_on, scl.scope_after_event
			from scope_config_locker scl
			where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.IsDefault = 0 and
			scl.scope_name = 'history';

			if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END

			insert into dbo.scope_config ( scope_name, scope_table, scope_description, scope_sync_on, scope_after_event )
			select top 1 scl.scope_name, scl.scope_table, scl.scope_description, scl.scope_sync_on, scl.scope_after_event
			from scope_config_locker scl
			where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.IsDefault = 0 and
			scl.scope_name = 'notification';

			if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END

			insert into dbo.scope_config ( scope_name, scope_table, scope_description, scope_sync_on, scope_after_event )
			select top 1 scl.scope_name, scl.scope_table, scl.scope_description, scl.scope_sync_on, scl.scope_after_event
			from scope_config_locker scl
			where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.IsDefault = 0 and
			scl.scope_name = 'search';

			if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END

			insert into dbo.scope_config ( scope_name, scope_table, scope_description, scope_sync_on, scope_after_event )
			select top 1 scl.scope_name, scl.scope_table, scl.scope_description, scl.scope_sync_on, scl.scope_after_event
			from scope_config_locker scl
			where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.IsDefault = 0 and
			scl.scope_name = 'workflow';

			if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END
		END

	set @v_DetailMessage = '   '+@v_stepname + ' scope_config ' + 'no hash-rows.'; 
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd, scope_chglog_row_cnt)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, @v_DetailMessage ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey, @v_rows);
	
	if (@v_rows = 0)
	begin
		--there is usually only one set of default rows. this may not need the same checks as the prior SQL.
		insert into dbo.scope_config ( scope_name, scope_table, scope_description, scope_sync_on, scope_after_event )
		select scl.scope_name, scl.scope_table, scl.scope_description, scl.scope_sync_on, scl.scope_after_event
		from scope_config_locker scl
					where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.scope_filter_hashcd is null and scl.IsDefault = 1;
		set @v_rows = @@rowcount;
		set @v_DetailMessage = '   '+@v_stepname + ' scope_config ' + 'default-rows.'; 
		
		
		if (@v_rows = 0)
		/*
			!!! If no rows are found here, ROLLBACK TRANSACTION because it probably means that SCOPE_CONFIG_LOCKER is incorrect. We should never fall through to the DEFAULT SCOPE_CONFIG_LOCKER
			!!! rows and find NO ROWS here. Exit with current @steperror
		*/
		BEGIN
		
			set @result = @steperror;

			insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
									 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
									 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd, scope_chglog_row_cnt)
			values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, @v_DetailMessage ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey, @v_rows);
		
			IF @@TRANCOUNT > 0
			ROLLBACK transaction;
	
			insert into dbo.scope_chglog_tbl 
				   (scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
					scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
					scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
					scope_chglog_dtm, scope_chglog_row_cnt)
			select scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
				   scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
				   scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
				   scope_chglog_dtm, scope_chglog_row_cnt
			from @v_ErrorLog
			order by scope_chglog_dtm asc;

			return @result;
		END;
		
		insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
								 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
								 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd, scope_chglog_row_cnt)
		values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, @v_DetailMessage ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey, @v_rows);
	end;
	
	
	/**************************************		
		S5: SCOPE_INFO RESTORE
		
			1. Try with Hashcd (filters) - non default rows - only for non master scope model
			2. Try without hashcd - non default rows
			3. Try default 	
	***************************************/
	
	disable trigger dbo.scope_info_sync_scope_timestamp_trigger ON dbo.scope_info;
	
	if (@p_TargetModel <> 1)
		BEGIN
			insert into scope_info ( sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_restore_count )
			select top 1 scl.sync_scope_local_id, scl.sync_scope_id, scl.sync_scope_name, scl.sync_scope_knowledge, scl.sync_scope_cleanup_knowledge, scl.sync_scope_datetime, scl.sync_scope_timestamp, 
				   scl.scope_restore_count
			from scope_info_locker scl
			where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId and scl.scope_filter_hashcd = v.TargetHashCd and v.TargetHashCd is not null) and
			scl.IsDefault = 0
			order by scl.sync_scope_datetime desc;
		END
	--Not handling for master scope model as it never comes with hash cd.

	set @v_rows = @@rowcount;
	set @v_DetailMessage = '   '+@v_stepname + ' scope_info ' + 'with hash-rows.'; 
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd, scope_chglog_row_cnt)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, @v_DetailMessage ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey, @v_rows);
	
	if (@v_rows = 0)
	begin
		set @v_rows = 0;

		if (@p_TargetModel <> 1)
			BEGIN
				insert into scope_info ( sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_restore_count )
				select top 1 scl.sync_scope_local_id, scl.sync_scope_id, scl.sync_scope_name, scl.sync_scope_knowledge, scl.sync_scope_cleanup_knowledge, scl.sync_scope_datetime, scl.sync_scope_timestamp, 
					   scl.scope_restore_count
				from scope_info_locker scl
				where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.scope_filter_hashcd is null and scl.IsDefault = 0
				order by scl.sync_scope_datetime desc;

				set @v_rows = @@rowcount;
			END
		else
			BEGIN
				--ensure that only 1 row per scope gets added. there might be a better way to do this.
				insert into scope_info ( sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_restore_count )
				select top 1 scl.sync_scope_local_id, scl.sync_scope_id, scl.sync_scope_name, scl.sync_scope_knowledge, scl.sync_scope_cleanup_knowledge, scl.sync_scope_datetime, scl.sync_scope_timestamp, 
					   scl.scope_restore_count
				from scope_info_locker scl
				where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and
					  scl.scope_filter_hashcd is null and scl.IsDefault = 0 and scl.sync_scope_name = 'aspnet'
				order by scl.sync_scope_datetime desc;

				set @v_rows = @@rowcount;

				insert into scope_info ( sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_restore_count )
				select top 1 scl.sync_scope_local_id, scl.sync_scope_id, scl.sync_scope_name, scl.sync_scope_knowledge, scl.sync_scope_cleanup_knowledge, scl.sync_scope_datetime, scl.sync_scope_timestamp, 
					   scl.scope_restore_count
				from scope_info_locker scl
				where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and
					  scl.scope_filter_hashcd is null and scl.IsDefault = 0 and scl.sync_scope_name = 'custom'
				order by scl.sync_scope_datetime desc;

				if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END

				insert into scope_info ( sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_restore_count )
				select top 1 scl.sync_scope_local_id, scl.sync_scope_id, scl.sync_scope_name, scl.sync_scope_knowledge, scl.sync_scope_cleanup_knowledge, scl.sync_scope_datetime, scl.sync_scope_timestamp, 
					   scl.scope_restore_count
				from scope_info_locker scl
				where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and
					  scl.scope_filter_hashcd is null and scl.IsDefault = 0 and scl.sync_scope_name = 'ektron'
				order by scl.sync_scope_datetime desc;

				if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END

				insert into scope_info ( sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_restore_count )
				select top 1 scl.sync_scope_local_id, scl.sync_scope_id, scl.sync_scope_name, scl.sync_scope_knowledge, scl.sync_scope_cleanup_knowledge, scl.sync_scope_datetime, scl.sync_scope_timestamp, 
					   scl.scope_restore_count
				from scope_info_locker scl
				where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and
					  scl.scope_filter_hashcd is null and scl.IsDefault = 0 and scl.sync_scope_name = 'history'
				order by scl.sync_scope_datetime desc;

				if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END

				insert into scope_info ( sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_restore_count )
				select top 1 scl.sync_scope_local_id, scl.sync_scope_id, scl.sync_scope_name, scl.sync_scope_knowledge, scl.sync_scope_cleanup_knowledge, scl.sync_scope_datetime, scl.sync_scope_timestamp, 
					   scl.scope_restore_count
				from scope_info_locker scl
				where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and
					  scl.scope_filter_hashcd is null and scl.IsDefault = 0 and scl.sync_scope_name = 'notification'
				order by scl.sync_scope_datetime desc;

				if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END

				insert into scope_info ( sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_restore_count )
				select top 1 scl.sync_scope_local_id, scl.sync_scope_id, scl.sync_scope_name, scl.sync_scope_knowledge, scl.sync_scope_cleanup_knowledge, scl.sync_scope_datetime, scl.sync_scope_timestamp, 
					   scl.scope_restore_count
				from scope_info_locker scl
				where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and
					  scl.scope_filter_hashcd is null and scl.IsDefault = 0 and scl.sync_scope_name = 'search'
				order by scl.sync_scope_datetime desc;

				if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END

				insert into scope_info ( sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_restore_count )
				select top 1 scl.sync_scope_local_id, scl.sync_scope_id, scl.sync_scope_name, scl.sync_scope_knowledge, scl.sync_scope_cleanup_knowledge, scl.sync_scope_datetime, scl.sync_scope_timestamp, 
					   scl.scope_restore_count
				from scope_info_locker scl
				where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and
					  scl.scope_filter_hashcd is null and scl.IsDefault = 0 and scl.sync_scope_name = 'workflow'
				order by scl.sync_scope_datetime desc;

				if (@v_rows = 0)
				BEGIN
					set @v_rows = @@rowcount;
				END
			END

		set @v_DetailMessage = '   '+@v_stepname + ' scope_info ' + 'no hash-rows.'; 
		
		insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
								 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
								 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd, scope_chglog_row_cnt)
		values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, @v_DetailMessage ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey, @v_rows);
	end;
	
	if (@v_rows = 0)
	begin
		--there are usually only one set of rows per scope for default rows.
		insert into scope_info ( sync_scope_local_id, sync_scope_id, sync_scope_name, sync_scope_knowledge, sync_scope_cleanup_knowledge, sync_scope_datetime, sync_scope_timestamp, scope_restore_count )
		select scl.sync_scope_local_id, scl.sync_scope_id, scl.sync_scope_name, scl.sync_scope_knowledge, scl.sync_scope_cleanup_knowledge, scl.sync_scope_datetime, scl.sync_scope_timestamp, 
			   scl.scope_restore_count
		from scope_info_locker scl
		where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId) and scl.scope_filter_hashcd is null and scl.IsDefault = 1
		order by scl.sync_scope_datetime desc;

		set @v_rows = @@rowcount;
		set @v_DetailMessage = '   '+@v_stepname + ' scope_info ' + 'default-rows.'; 
		
		insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							     scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							     scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd, scope_chglog_row_cnt)
		values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, @v_DetailMessage ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey, @v_rows);
	end;	

	/**************************************
		S5: SCOPE_INFO_HISTORY RESTORE - skipping this as it is not required.
	***************************************/
	
	enable trigger dbo.scope_info_sync_scope_timestamp_trigger ON dbo.scope_info;
	
	
	/**************************************
		S5: SCOPE_LOGINFO RESTORE - skipping this as it is not required.
	***************************************/
	
	
	/**************************************		
		S5: SCOPE_TABLE_MAP RESTORE
		
		Hash codes do not apply to this table	
	***************************************/

	insert into scope_table_map ( scope_name, table_name, table_order )
	select scl.scope_name, scl.table_name, scl.table_order
	from scope_table_map_locker scl
	where exists (select 'x' from @v_TargetModel v where scl.scope_manager_id = v.TargetModelId);
	
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
		
	/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		Step6: Set target model as current model
	  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/
	set @steperror = -6;
	set	@v_stepname='Step6: Set target model as current model';
	
	insert into scope_locker_history ( scope_id, scope_filter_hashcd ) values ( @p_TargetModel, @v_EntityIdHashKey );
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
	
	end try
	
	/***********************************************	
		CATCH error processing	
	 ***********************************************/
	
	begin catch
	
		select @v_errorcd = ERROR_NUMBER(),
			@v_errormsg = ERROR_MESSAGE(),
			@v_currentlyActiveScopeModel = coalesce(@v_currentlyActiveScopeModel,0)
			
		if @v_errorcd is not null 
			BEGIN
				if (@v_errorcd > 0)
					set @result = @v_errorcd*-1;
				else
					set @result = @v_errorcd;
			end
		else set @result = @steperror;	
		
		-- If Error is encountered in the procedure, rollback all changes
		IF @@TRANCOUNT > 0
		    ROLLBACK TRANSACTION;
		    
		
		insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
								 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
								 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
		values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @v_errorcd, @v_errormsg ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
			
	end catch;
	
	-- If transaction is still active, no errors encountered, commit all changes
	
	IF @@TRANCOUNT > 0
    commit transaction;
	
	/*
		Insert all log rows to the logging table. Logging done like this to enable AUTONOMOUS "like" processing of log rows. Placing log rows into a
		table variable allows rows to persist when transaction is rolled back (ON ERROR).
	*/
	
	insert into dbo.scope_chglog_tbl 
		   (scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
			scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
			scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
			scope_chglog_dtm, scope_chglog_row_cnt)
	select scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
		   scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
		   scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
		   scope_chglog_dtm, scope_chglog_row_cnt
	from @v_ErrorLog
	order by scope_chglog_dtm asc;
	
	return @result;	
end
go
